import os
from enum import Enum
from pathlib import Path

from PySide2.QtCore import QFile
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QDialog, QListWidgetItem, QHBoxLayout, QWidget
from axipy import PointStyle, LineStyle, PolygonStyle, TextStyle, StyleButton, Style

from StyleTools.controls.compositeStyle.ui.StyleComposite.helper_wigets import StylePoint, PaintStyleWidget, StylePen, \
    StylePolygon


#from AxyPy.Style.StyleComposite.helper_wigets import StylePoint, StylePen, StylePolygon, PaintStyleWidget


class TypeStyleGeometry(Enum):
    Point=1
    Line=2
    Polygon=3
    Text=4
    CollectionPoint=5
    CollectionLine = 6
    CollectionPolygon=6
    CollectionText=7

class DlgCompositeStyle(QDialog):
    def __init__(self,style,parent=None):
        super().__init__(parent)
        self.__result_style=None
        self.__style=style
        self.isOk=False
        curent_path = Path(__file__).parent
        path_icon = curent_path.joinpath("icons")
        self.ui=self.load_ui("DlgcompsiteStyle1.ui",parent)
        path_add=str(path_icon.joinpath('add_32.png'))
        path_del = str(path_icon.joinpath('delete_32.png'))
        path_up = str(path_icon.joinpath('up_32.png'))
        path_down = str(path_icon.joinpath('down_32.png'))
        self.ui.finished.connect(self.__close)
        self.ui.tb_add.setIcon(QIcon(path_add))
        self.ui.tb_del.setIcon(QIcon(path_del))
        self.ui.tb_up.setIcon(QIcon(path_up))
        self.ui.tb_down.setIcon(QIcon(path_down))
        self.ui.tb_add.clicked.connect(self.__add_style)
        self.ui.tb_del.clicked.connect(self.__delStyle)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_close.clicked.connect(self.__close)
        self.__prepareStyle(style)
        self.ui.listWidget.itemSelectionChanged.connect(self.__change_selection)

    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def show(self):
        self.ui.show()
    def exec_(self):
        self.ui.exec_()
    def __prepareStyle(self,style):
        self.__type_base=self.__getTypeStyle(style)

        if self.__type_base is not None:
            self.__initWigets([style])
            return
        str_mi_style=style.to_mapinfo()
        list_styles=str_mi_style.split(";")
        if len(list_styles)==0:
            list_styles = str_mi_style.split("|")
        if len(list_styles)>0:
            styles=[]
            for mi_stl in list_styles:
                styles.append(Style.from_mapinfo(mi_stl))
            self.__type_base=self.__getTypeStyle(styles[0])
            self.__initWigets(styles)
            if len(styles)>1:
                self.ui.tb_del.setEnabled(True)
            return
    def __getTypeStyle(self,style):
        self.__type_base = None
        if isinstance(style, PointStyle):
            return TypeStyleGeometry.Point
        if isinstance(style, LineStyle):
            return TypeStyleGeometry.Line
        if isinstance(style, PolygonStyle):
            return   TypeStyleGeometry.Polygon
        if isinstance(style, TextStyle):
            return  TypeStyleGeometry.Text
        return None
    def __initWigets(self,list_w_styles):
        self.__widgetPaint=PaintStyleWidget(None,self.ui)
        self.__widgetPaint.setEnabled(False)
        self.__stylePaint=None
        if self.__type_base==TypeStyleGeometry.Point:
            self.__stylePaint=StylePoint(self.__style)
        if self.__type_base==TypeStyleGeometry.Line:
            self.__stylePaint = StylePen(self.__style)
        if self.__type_base == TypeStyleGeometry.Polygon:
            self.__stylePaint = StylePolygon(self.__style)

        self.ui.layout_style.addWidget(self.__widgetPaint)
        if self.__stylePaint is not None:
            self.__widgetPaint.setObjectStyle(self.__stylePaint)
            self.__widgetPaint.repaint()
        for item_style in list_w_styles:
            '''
            item = QListWidgetItem()
            item_widget=StyleButton(item_style,self.ui)
            self.ui.listWidget.addItem(item)
            self.ui.listWidget.setItemWidget(item, item_widget)
            '''
            self.__addStyleWidget(item_style)
        self.ui.listWidget.setCurrentRow(0)
    def __addStyleWidget(self,style):
        item = QListWidgetItem()
        item_widget = QWidget()
        widget_style = StyleButton(style)
        widget_style.style_changed.connect(self.__change_style)
        item_layout = QHBoxLayout()
        item_layout.addWidget(widget_style)
        item_widget.setLayout(item_layout)
        item.setSizeHint(item_widget.sizeHint())
        self.ui.listWidget.addItem(item)
        self.ui.listWidget.setItemWidget(item, item_widget)

    def __change_style(self):
        self.__getCurentStyle()
    def __getCurentStyle(self):
        count_styles=self.ui.listWidget.count()
        styles_mi=[]
        for i in range(count_styles):
            itemWidget=self.ui.listWidget.item(i)
            widget = self.ui.listWidget.itemWidget(itemWidget)
            children=widget.findChild(StyleButton)
            styles_mi.append(children.style.to_mapinfo())
        common_style=';'.join(styles_mi)
        self.__widgetPaint.updateStyle(Style.from_mapinfo(common_style))
    def __add_style(self):
        mi_style=None
        if self.__type_base==TypeStyleGeometry.Point:
            mi_style='Symbol (33,0,12,"Map Symbols",0,0) '
        if self.__type_base==TypeStyleGeometry.Line:
            mi_style='Pen (1, 2, 0)'
        if self.__type_base==TypeStyleGeometry.Polygon:
            mi_style='Pen (1, 2, 0) Brush (2, 16777215, 16777215)'
        if mi_style is not None:
            style=Style.from_mapinfo(mi_style)
            self.__addStyleWidget(style)
            self.__getCurentStyle()
        if self.ui.listWidget.count() > 1:
            self.ui.tb_del.setEnabled(True)
        self.__change_selection()
    def __delStyle(self):
        row_select = self.ui.listWidget.currentRow()
        self.ui.listWidget.takeItem(row_select)
        if self.ui.listWidget.count()==1:
            self.ui.tb_del.setEnabled(False)
            self.ui.tb_up.setEnabled(False)
            self.ui.tb_down.setEnabled(False)
        self.__getCurentStyle()
        self.__change_selection()
    def __change_selection(self):
        row_select = self.ui.listWidget.currentRow()
        count=self.ui.listWidget.count()
        if row_select==0:
            self.ui.tb_up.setEnabled(False)
            if count>1:
                self.ui.tb_down.setEnabled(True)
            return
        if row_select==(self.ui.listWidget.count()-1):
            self.ui.tb_down.setEnabled(False)
            if count>1:
                self.ui.tb_up.setEnabled(True)
            return
        self.ui.tb_up.setEnabled(True)
        self.ui.tb_down.setEnabled(True)
    def __run(self):
        count_styles = self.ui.listWidget.count()
        styles_mi = []
        for i in range(count_styles):
            itemWidget = self.ui.listWidget.item(i)
            widget = self.ui.listWidget.itemWidget(itemWidget)
            children = widget.findChild(StyleButton)
            styles_mi.append(children.style.to_mapinfo())
        self.__result_style = ';'.join(styles_mi)
        self.isOk=True
        self.ui.close()
    def __close(self):
        if not self.isOk:
            self.__result_style=None
        self.ui.close()

    @property
    def resultStyle(self):
        return self.__result_style








