from PySide2.QtCore import Signal
from PySide2.QtGui import QPainter, Qt
from PySide2.QtWidgets import QWidget
from axipy import Point, GeometryCollection, Line, Pnt, Polygon, PointStyle, LineStyle, PolygonStyle, TextStyle, Style




#from AxyPy.Style.StyleComposite.DlgComposite import DlgCompositeStyle


class StylePolygon:
    def __init__(self,style,parent=None):
        self.__parent=parent
        self.__style=style
    def setParent(self,parent):
        self.__parent=parent
    def setStyle(self,style):
        self.__style=style
    @property
    def style(self):
        return self.__style
    def __getGeometry(self):
        size_parent = self.__parent.size()
        try:
            width_line=self.__style.border.width
        except:
            width_line=9
        if width_line>10:
            ''' ширина в пунктах '''
            width_pt=(width_line-10)/10
            width_line=int(width_pt*0.75292857248934+0.5)
        dl=int(width_line/2+0.5)+1
        pt1=Pnt(dl,dl)
        pt2=Pnt(dl,size_parent.height()-dl)
        pt3=Pnt(size_parent.width()-dl,size_parent.height()-dl)
        pt4=Pnt(size_parent.width()-dl,dl)
        #pt5=Pnt(size_parent.width()/+dl,3+)
        poly=Polygon([pt1,pt2,pt3,pt4])
        return poly

    def paint(self):
        if self.__style is None:
            return
        size_parent = self.__parent.size()
        geo_obj = self.__getGeometry()
        qp = QPainter()
        qp.begin(self.__parent)
        qp.fillRect(0, 0, size_parent.width(), size_parent.height(), Qt.white)
        self.__style.draw(geo_obj, qp)
        qp.end()

class StylePen:
    def __init__(self,style,parent=None):
        self.__parent=parent
        self.__style=style
    def setParent(self,parent):
        self.__parent=parent
    def setStyle(self,style):
        self.__style=style

    @property
    def style(self):
        return self.__style
    def __getGeometry(self):
        size_parent = self.__parent.size()
        pt1=Pnt(0,0)
        pt2=Pnt(size_parent.width(),size_parent.height())
        geo_line=Line(pt1,pt2)
        pt1_1=Pnt(size_parent.width(),0)
        pt2_2=Pnt(0,size_parent.height())
        geo_line_2 = Line(pt1_1, pt2_2)
        geo_col=GeometryCollection()
        geo_col.append(geo_line)
        geo_col.append(geo_line_2)
        return geo_col
    def paint(self):
        if self.__style is None:
            return
        size_parent=self.__parent.size()
        geo_obj = self.__getGeometry()
        qp = QPainter()
        qp.begin(self.__parent)
        qp.fillRect(0, 0, size_parent.width(), size_parent.height(), Qt.white)
        self.__style.draw(geo_obj, qp)
        qp.end()

class StylePoint:
    def __init__(self,style,parent=None):
        self.__parent=parent
        self.__style=style
    def setStyle(self,style):
        self.__style=style

    @property
    def style(self):
        return self.__style
    def paint(self):
        if self.__style is None:
            return
        size_parent=self.__parent.size()
        point = Point(size_parent.width()/2,size_parent.height()/2)
        qp = QPainter()
        print("Begin")
        qp.begin(self.__parent)
        qp.fillRect(0, 0, size_parent.width(), size_parent.height(), Qt.white)
        self.__style.draw(point, qp)
        qp.end()
        print("end")
    def setParent(self,parent):
        self.__parent=parent

class PaintStyleWidget(QWidget):
    channged_style = Signal(str)
    def __init__(self,style_object,parent=None):
        super().__init__(parent)
        self.__parent=parent
        self.__style_object=None
        self._possibly_dlg=True
        if style_object is not None:
            self.__style_object=style_object
            self.__style_object.setParent(self)
    def setObjectStyle(self,style):
        self.__style_object=style
        self.__style_object.setParent(self)
    def setDisableDlg(self,value):
        self._possibly_dlg=value
    @property
    def style(self):
        return self.__style_object.style
    def paintEvent(self, event):
        if self.__style_object is not None:
            self.__style_object.paint()
        else:
            qp = QPainter()
            qp.begin(self)
            size = self.size()
            qp.fillRect(0, 0, size.width(), size.height(), Qt.white)
            qp.end()

    def mousePressEvent(self, event):
        print("clicked")
        if not self._possibly_dlg:
            return
        #from AxyPy.Style.StyleComposite.DlgComposite import DlgCompositeStyle
        from StyleTools.controls.compositeStyle.ui.StyleComposite.DlgComposite import DlgCompositeStyle
        dlg_copmposite_style=DlgCompositeStyle(self.__style_object.style,self)
        dlg_copmposite_style.exec_()
        result_style_str=dlg_copmposite_style.resultStyle
        if result_style_str is None:
            return
        new_style=Style.from_mapinfo(result_style_str)
        self.updateStyle(new_style)
        self.channged_style.emit(result_style_str)

    def updateStyle(self,style):
        self.__style_object.setStyle(style)
        self.repaint()
    @staticmethod
    def factoryStyleWidget(type_object,style,parent=None):
        stylePaint = None
        #from AxyPy.Style.StyleComposite.DlgComposite import TypeStyleGeometry
        from StyleTools.controls.compositeStyle.ui.StyleComposite.DlgComposite import TypeStyleGeometry

        if type_object == TypeStyleGeometry.Point:
            stylePaint = StylePoint(style)
        if type_object == TypeStyleGeometry.Line:
            stylePaint = StylePen(style)
        if type_object == TypeStyleGeometry.Polygon:
            stylePaint = StylePolygon(style)
        if stylePaint is None:
            return None
        widgetPaint = PaintStyleWidget(None, parent)
        widgetPaint.setObjectStyle(stylePaint)
        return widgetPaint
    @staticmethod
    def getTypeStyle(style):
        #from AxyPy.Style.StyleComposite.DlgComposite import TypeStyleGeometry
        from StyleTools.controls.compositeStyle.ui.StyleComposite.DlgComposite import TypeStyleGeometry
        if isinstance(style, PointStyle):

            return TypeStyleGeometry.Point
        if isinstance(style, LineStyle):
            return TypeStyleGeometry.Line
        if isinstance(style, PolygonStyle):
            return TypeStyleGeometry.Polygon
        if isinstance(style, TextStyle):
            return  TypeStyleGeometry.Text
        ''' Возможно композитный стиль '''
        styles=copmpositeStyleInList(style)
        if styles is None:
            return None
        return  PaintStyleWidget.getTypeStyle(styles[0])

def copmpositeStyleInList(style:Style):
    mi_style=style.to_mapinfo()
    list_item_str=mi_style.split(";")
    list_style = []
    if len(list_item_str)==0:
        list_style.append(style)
    else:

        for item in list_item_str:
            cur_style=Style.from_mapinfo(item)
            if cur_style is not None:
                list_style.append(cur_style)
    if len(list_style)>0:
        return list_style
    return None
