from pathlib import Path
from typing import Optional

from PySide2.QtWidgets import QFileDialog
from axipy import attr, provider_manager, mainwindow, CurrentSettings, tr


def createTempTab(name,cs,hidden=False):
    definition = {
        'src': '',
        'schema': attr.schema(
            attr.integer('id'),
            coordsystem=cs
        ),
        'hidden': hidden

    }
    #if not hidden:
    table = provider_manager.create(definition)
    #else:
    #    definition['hidden']=True
    #    table = provider_manager.open(definition)


    table.name=name
    return table
def openTabSqlite(path_db,name_tab):
    tab= provider_manager.open_hidden({'src':path_db,'dataobject': name_tab})
    #tab= provider_manager.openfile(path_db,dataobject =name_tab)
    return tab
def open_file_dialogNew(caption=None,filter_arg: str = None) -> Optional[Path]:
    """
    Открывает диалог выбора файла. Возвращает путь к выбранному файлу.
    Если нет главного окна Аксиомы, спрашивает путь к файлу в консоли.

    Args:
        filter_arg: Типы файлов. Например: ``'MapInfo Tab (*.tab);;Таблицы Excel (*.xls *.xlsx)'``.

    """
    if mainwindow.is_valid:
        params = {
            "parent": mainwindow.qt_object(),
            "dir": str(CurrentSettings.LastOpenPath)
        }
        if caption is not None:
            params['caption']=caption
        if filter_arg is not None:
            params["filter"] = filter_arg
        if not CurrentSettings.UseNativeFileDialog:
            params["options"] = QFileDialog.DontUseNativeDialog
        result = QFileDialog.getOpenFileName(**params)[0]
    else:
        result = input(tr("Введите путь к файлу: "))

    if result:
        result_str = result
        result = Path(result)
        if not (result.is_file() or result.exists()):
            raise FileNotFoundError(f"File not found: \n{result_str}")
        else:
            return result
    else:
        return None
def save_file_dialogNew(caption=None,filter_arg: str = None) -> Optional[Path]:
    """
    Открывает диалог сохранения файла. Возвращает выбранный путь сохранения.
    Если нет главного окна Аксиомы, спрашивает путь к файлу в консоли.

    Args:
        filter_arg: Типы файлов. Например: ``'MapInfo Tab (*.tab);;Таблицы Excel (*.xls *.xlsx)'``.

    """
    if mainwindow.is_valid:
        params = {
            "parent": mainwindow.qt_object(),
            "dir": str(CurrentSettings.LastSavePath)
        }
        if caption is not None:
            params['caption']=caption
        if filter_arg is not None:
            params["filter"] = filter_arg
        if not CurrentSettings.UseNativeFileDialog:
            params["options"] = QFileDialog.DontUseNativeDialog
        result = QFileDialog.getSaveFileName(**params)[0]
    else:
        result = input(tr("Введите путь сохранения файла: "))

    if result:
        return Path(result)
    else:
        return None


