import os
from pathlib import Path

import axipy
from PySide2 import QtSvg
from PySide2.QtCore import QFile, Qt, QByteArray, Signal
from PySide2.QtGui import QIcon, QPixmap
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QSpacerItem, QSizePolicy, QLabel
from axipy import Rect

from StyleTools.controls.pointStyleEditor.ui.add_tools.DbTools.SqliteDataSource import DbSqliteDataSource
from StyleTools.controls.pointStyleEditor.ui.add_tools.DbTools.dbSymbols import DbSymbols
from StyleTools.controls.pointStyleEditor.ui.add_tools.helper_tab import open_file_dialogNew, save_file_dialogNew


class WidgetDbStyle(QWidget):
    edit_signal = Signal(str)
    def __init__(self,parent=None):
        super().__init__(parent)
        self._dbSpatial=None
        self.__db_symbol=None
        self.__ColumnCount = 3
        self.ui=self.load_ui("widgetDbStyle1.ui",parent)
        curent_path = Path(__file__).parent
        path_icon = curent_path.joinpath("icons")
        path_add = str(path_icon.joinpath('add.png'))
        path_del=str(path_icon.joinpath('del.png'))
        path_edit = str(path_icon.joinpath('edit.png'))

        self.ui.tb_add.setIcon(QIcon(path_add))
        self.ui.tb_del.setIcon(QIcon(path_del))
        self.ui.tb_edit.setIcon(QIcon(path_edit))
        self.ui.tb_create.clicked.connect(self.__createDbStyle)
        self.ui.pb_open.clicked.connect(self.__openDbStyle)

        self.ui.tb_edit.clicked.connect(self.__run_edit)
        self.ui.tb_del.clicked.connect(self.__run_del)
        self.ui.tableWidget.itemPressed.connect(self.__run_edit)
        self.__version_need = axipy.Version.compare(6, 0)
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def show(self):
        self.ui.show()
    def __createDbStyle(self):
        filter="'Gpkg (*.gpkg);;"
        path_db=save_file_dialogNew("Создать базу символов",filter)
        if path_db is not None:
            self.__str_path_db=str(path_db)
            self.ui.ln_path_db.setText(self.__str_path_db)
            self.__createNewDb(self.__str_path_db)
            self.ui.groupBox_styles.setEnabled(True)
    def __openDbStyle(self):

        filter = "'Gpkg (*.gpkg);;"
        path_db = open_file_dialogNew("Открыть базу символов", filter)
        if path_db is None:
            self.ui.groupBox_styles.setEnabled(False)
            return
        self.__str_path_db = str(path_db)
        self.ui.ln_path_db.setText(self.__str_path_db)
        self.__openDB(self.__str_path_db)
        self.ui.groupBox_styles.setEnabled(True)
    def __openDB(self,path_db):
        if self._dbSpatial is not None:
            self._dbSpatial.close()
        self._dbSpatial = DbSqliteDataSource()
        self._dbSpatial.open(path_db)
        self.__db_symbol = DbSymbols(self._dbSpatial.DbSqlite)
        self.__updateStyleTable()
    def __createNewDb(self,path_db):

        self.ui.tableWidget.clear()
        if self._dbSpatial is not None:
            self._dbSpatial.close()
        if Path(path_db).exists():
            os.remove(path_db)
        self._dbSpatial=DbSqliteDataSource()
        self._dbSpatial.create(path_db)
        self._dbSpatial.createCatalog()
        self.__db_symbol=DbSymbols(self._dbSpatial.DbSqlite)
        self.__setEnabled(True, False, False)
    def closeDb(self):
        if self._dbSpatial is not None:
            self._dbSpatial.close()
        self._dbSpatial=None
    @property
    def symbolDb(self):
        return self.__db_symbol
    @property
    def spatialDb(self):
        return self._dbSpatial
    @property
    def IsOpenDb(self):
        if self._dbSpatial is None:
            return False
        return True
    def getMaxId(self):
        maxId=self.__db_symbol.lastId()
        name_symbol="symbol_"+"{:03d}".format(maxId)
        return name_symbol
    @property
    def pathDb(self):
        if not self.IsOpenDb:
            return None
        return self.__str_path_db
    def InsertTable(self,name_style,bound:Rect,svg):
        self.__db_symbol.insert(name_style,bound,svg)
        self.__updateStyleTable()
    def __updateStyleTable(self):
        self.ui.tableWidget.clearContents()
        self.ui.tableWidget.setColumnCount(self.__ColumnCount)

        all_symbol = self.__db_symbol.loadAll()
        all_symbol_count = len(all_symbol)
        if all_symbol_count==0:
            self.__setEnabled(True,False,False)
            return
        row, col = divmod(all_symbol_count, self.__ColumnCount)
        count_row = row
        if col > 0:
            count_row = count_row + 1
        self.ui.tableWidget.setRowCount(count_row)
        for i, symbol in enumerate(all_symbol):
            row, col = divmod(i, self.__ColumnCount)
            if not(self.__version_need>=0):
                item = self.__createWidgetItemSvg(symbol)
            else:
                item=self.__createWidgetItemPng(symbol)
            # self.ui.tableWidget.setItem(row, col, item)
            self.ui.tableWidget.setCellWidget(row, col, item)
        self.ui.tableWidget.resizeRowsToContents()
        self.__setEnabled(True, True, True)
    def __createWidgetItemSvg(self,symbol):


        cell_widget = QWidget()
        cell_widget.setStyleSheet('border-style: solid; border-width: 0px; border-color: black;')
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout_svg = QHBoxLayout()
        layout_svg.setContentsMargins(0, 0, 0, 0)
        horizantSpacer_1 = QSpacerItem(20, 40, QSizePolicy.Fixed, QSizePolicy.Expanding)
        horizantSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Fixed, QSizePolicy.Expanding)
        verticalSpacer = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Expanding)
        label_name = QLabel()

        label_name.setAlignment(Qt.AlignCenter)
        label_name.setText(symbol.Name)
        label_name.setFixedHeight(14)
        svgWidget = QtSvg.QSvgWidget()
        svgWidget.load(symbol.symbol)
        #svgWidget.setAlignment(Qt.AlignCenter)
        svgWidget.setFixedHeight(44)
        svgWidget.setFixedWidth(44)
        layout_svg.addSpacerItem(horizantSpacer_1)
        layout_svg.addWidget(svgWidget)
        layout_svg.addSpacerItem(horizantSpacer_2)
        layout.addWidget(label_name)

        layout.addLayout(layout_svg)
        layout.addSpacerItem(verticalSpacer)
        #layout.addWidget(svgWidget)
        cell_widget.setLayout(layout)
        return cell_widget
    def __createWidgetItemPng(self,symbol):


        cell_widget = QWidget()
        cell_widget.setStyleSheet('border-style: solid; border-width: 0px; border-color: black;')
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout_svg = QHBoxLayout()
        #layout_svg.setContentsMargins(0, 0, 0, 0)
        horizantSpacer_1 = QSpacerItem(20, 40, QSizePolicy.Fixed, QSizePolicy.Expanding)
        horizantSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Fixed, QSizePolicy.Expanding)
        verticalSpacer = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Expanding)
        label_name = QLabel()

        label_name.setAlignment(Qt.AlignCenter)
        label_name.setText(symbol.Name)
        label_name.setFixedHeight(14)
        pixmap = QPixmap()
        pixmap.loadFromData( symbol.symbol)
        resize_image=pixmap.scaled(40,40)
        imageWidget=QLabel()
        imageWidget.setAlignment(Qt.AlignCenter)
        imageWidget.setFixedHeight(44)
        imageWidget.setFixedWidth(44)
        layout_svg.addSpacerItem(horizantSpacer_1)
        layout_svg.addWidget(imageWidget)
        layout_svg.addSpacerItem(horizantSpacer_2)
        imageWidget.setPixmap( resize_image)
        imageWidget.setFixedHeight(44)
        imageWidget.setFixedWidth(44)
        layout.addWidget(label_name)

        layout.addLayout(layout_svg)
        layout.addSpacerItem(verticalSpacer)
        #layout.addWidget(layout_svg)
        cell_widget.setLayout(layout)
        return cell_widget
    def __setEnabled(self,add_btn,del_btn,edit_btn):
        self.ui.tb_add.setEnabled(add_btn)
        self.ui.tb_del.setEnabled(del_btn)
        self.ui.tb_edit.setEnabled(edit_btn)
    def __run_edit(self):
        cur_widget = self.__getCurentWidget()
        if cur_widget is None:
            return
        label_style = cur_widget.findChildren(QLabel)
        name_style = label_style[0].text()
        self.edit_signal.emit(name_style)

    def __run_del(self):
        cur_widget=self.__getCurentWidget()
        if cur_widget is None:
            return
        label_style=cur_widget.findChildren(QLabel)
        name_style=label_style[0].text()
        self._dbSpatial.deleteSymbolLayer(name_style)
        self.__updateStyleTable()
        jkl=0
    def __getCurentWidget(self):
        col = self.ui.tableWidget.currentColumn()
        row = self.ui.tableWidget.currentRow()
        cur_widget=self.ui.tableWidget.cellWidget(row,col)
        jkl=0
        return cur_widget
