import os
from pathlib import Path

import axipy
from PySide2.QtCore import Signal, QFile
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QFileDialog
from axipy import mainwindow, CurrentSettings

from StyleTools.controls.pointStyleEditor.ui.add_tools.RenderTools import renderTabToSvg, renderTabToPng
from StyleTools.controls.pointStyleEditor.ui.add_tools.helper_tab import openTabSqlite


class CopyStyle(QWidget):
    save_signal = Signal(list)
    def __init__(self,cls_db_style,parent=None):
        super().__init__(parent)
        self.__cls_dbStyleWidget=cls_db_style
        self.__auto_sync=False
        self.ui=self.load_ui("widgetCopy.ui",parent)
        curent_path = Path(__file__).parent
        path_icon = curent_path.joinpath("icons")
        path_saveas = str(path_icon.joinpath('saveas.png'))
        self.ui.tb_saveas.setIcon(QIcon(path_saveas))
        self.ui.com_format.currentIndexChanged.connect(self.__change_format)
        self.ui.tb_saveas.clicked.connect(self.__selectOutFolder)
        self.ui.pb_copy.clicked.connect(self.__copy_all_symbol)
        self.ui.com_format.setCurrentIndex(0)
        self.__version_need = axipy.Version.compare(6,0)
        if not(self.__version_need <= 0):
            ''' Версия ниже 6.0
            формат хранения только png 
            '''
            self.ui.com_format.setCurrentIndex(1)
            self.ui.com_format.setEnabled(False)
        self.ui.ch_auto.stateChanged.connect(self.__ch_auto)
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __selectOutFolder(self):
        caption="Директория стилей точечных объектов"
        params = {
            "parent": mainwindow.qt_object(),
            "dir": str(CurrentSettings.LastOpenPath),
            "options":QFileDialog.ShowDirsOnly,
            'caption' : caption
        }
        result = QFileDialog.getExistingDirectory(**params)
        if len(result)==0:
            return
        self.ui.ln_folder.setText(result)
        self.ui.ln_folder.setCursorPosition(0)
        if self.__cls_dbStyleWidget.IsOpenDb:
            self.ui.pb_copy.setEnabled(True)
        self.ui.ch_auto.setEnabled(True)
    def __change_format(self,index):
        jkl=0
    def __ch_auto(self,arg):
        self.__auto_sync =self.ui.ch_auto.isChecked()
        path_out=self.ui.ln_folder.text()
        if not self.__auto_sync:
            path_out=None
        self.save_signal.emit([path_out,self.ui.com_format.currentText()])
    @property
    def isAutoSync(self):
        if self.__auto_sync:
            return self.ui.ln_folder.text()
        return None
    def __copy_all_symbol(self):
        symbol_db=self.__cls_dbStyleWidget.symbolDb
        path_db=self.__cls_dbStyleWidget.pathDb
        all_symbol=symbol_db.loadAll()
        format_save=self.ui.com_format.currentText()
        path_out=Path(self.ui.ln_folder.text())
        for style_symbol in all_symbol:
            tab=openTabSqlite(path_db,style_symbol.Name)
            bound=style_symbol.rect
            bufeer_qt=None
            if format_save=='svg':
                parh_out_file=str(path_out.joinpath(style_symbol.Name+".svg"))
                bufeer_qt=renderTabToSvg(tab,bound)
            else:
                parh_out_file = str(path_out.joinpath(style_symbol.Name + ".png"))
                bufeer_qt=renderTabToPng(tab, bound)
            tab.close()

            if bufeer_qt is not None:
                out_data = bufeer_qt.data()
                with open(parh_out_file, "wb") as binary_file:
                    # Write bytes to file
                    binary_file.write(out_data)


